"""Contains the interfaces that goals will implement."""
import abc
import typing as t
from enum import Enum


class Language(Enum):
    """The languages that the linters can check."""

    TERRAFORM = "terraform"
    PYTHON = "python"


class Linter(abc.ABC):
    """The interface that all linters will implement."""

    name: str
    parallel_run: bool
    language: Language
    weight: int = 0

    def __init__(self, args: t.Optional[t.List[str]] = None):
        """Will initialize the linter.

        Args are passed through to the linter.
        """
        if args is None:
            args = []
        self._args = args

    @abc.abstractmethod
    def run(self):
        """Will run the linter.

        Depending on the linter this may have side effects
        """
        raise NotImplementedError

    @abc.abstractmethod
    def check(self):
        """Will run the linter in check mode.This should NEVER change any files."""
        raise NotImplementedError


class Tester(abc.ABC):
    """The interface that all Testers will implement."""

    name: str
    language: Language

    def __init__(self, args: t.Optional[t.List[str]] = None, test_dir=None):
        """Will initialize the Tester.

        Args are passed through to the Tester.
        """
        if args is None:
            args = []
        self._args = args
        self._test_dir = test_dir

    @abc.abstractmethod
    def run(self):
        """Will run the tester."""
        raise NotImplementedError


class Deployers(abc.ABC):
    """The interface that all deployers will implement."""

    name: str

    def __init__(self, args: t.Optional[t.List[str]] = None, pyproject_loc: t.Optional[str] = None):
        """Will initialize the deployer.

        Args are passed through to the deployer.
        """
        if args is None:
            args = []
        self._args = args
        self._pyproject_loc = pyproject_loc

    @abc.abstractmethod
    def plan(self):
        """Will run the deployer in plan mode."""
        raise NotImplementedError

    @abc.abstractmethod
    def build(self):
        """Will build anything the deployer needs."""
        raise NotImplementedError

    @abc.abstractmethod
    def run(self):
        """Will run the deployer."""
        raise NotImplementedError



"""Contains all the implemented linters."""
import typing as t

from py_mono_tools.config import consts, GREEN, logger, RED, RESET
from py_mono_tools.goals.interface import Language, Linter


CHECK_STRING = " check"
MAX_LINE_LENGTH = 120


def _run(linter: str, args: t.List[str]) -> t.Tuple[str, int]:
    log_format = "\n" + "#" * 20 + "  {}  " + "#" * 20 + "\n"
    logs = ""

    logger.debug("Running %s: %s", linter, args)
    if len(args) > 0 and "docker" == args[0] and consts.CURRENT_BACKEND.name == "docker":  # type: ignore
        logger.debug("Bypassing docker backend for system backend. Linter: %s", linter)
        return_code, returned_logs = consts.BACKENDS["system"]().run(args)  # type: ignore
    else:
        return_code, returned_logs = consts.CURRENT_BACKEND.run(args)  # type: ignore
    logger.debug("%s return code: %s", linter, return_code)

    color = GREEN if return_code == 0 else RED
    logs += color
    logs += log_format.format(linter + " start")
    logs = logs + returned_logs
    logs += color
    logs += log_format.format(linter + " end")
    logs += RESET

    return logs, return_code


class Bandit(Linter):
    """
    Bandit linter.

    Helps find common python security issues
    https://bandit.readthedocs.io/en/latest/
    """

    name: str = "bandit"
    parallel_run: bool = True
    language = Language.PYTHON

    def run(self):
        """Will run the bandit linter recursively."""
        args = [
            "bandit",
            "-r",
            consts.EXECUTED_FROM,
            *self._args,
        ]
        return _run(self.name, args)

    def check(self):
        """Will run the bandit linter recursively."""
        return self.run()


class Black(Linter):
    """
    Black linter.

    Formats your code the correct way.
    https://black.readthedocs.io/en/stable/
    """

    name: str = "black"
    parallel_run: bool = False
    weight: int = 99
    language = Language.PYTHON

    def run(self):
        """
        Will run the black linter.

        NOTE: This WILL modify your files.
        """
        args = [
            "black",
            consts.EXECUTED_FROM,
            *self._args,
        ]
        return _run(self.name, args)

    def check(self):
        """
        Will run the black linter in check mode.

        NOTE: This will NOT modify your files.
        """
        args = [
            "black",
            "--check",
            consts.EXECUTED_FROM,
            *self._args,
        ]
        return _run(self.name + CHECK_STRING, args)


class Flake8(Linter):
    """
    Flake8 linter.

    Style checker for Python code.

    Flake8 runs: pycodestyle, pyflakes, and mccabe
    https://flake8.pycqa.org/en/latest/faq.html#why-does-flake8-use-ranges-for-its-dependencies

    WARNING: Flake8 must be run with the SAME version of python that your code will run on!

    The default max complexity is 10.
    The default max line length is 120.
    """

    name: str = "flake8"
    parallel_run: bool = False
    language = Language.PYTHON

    def __init__(self, args: t.Optional[t.List[str]] = None):
        """Will set the max complexity and max line length."""
        no_max_comp = True
        no_max_line = True
        if args is None:
            args = []

        for arg in args:
            if "--max-complexity" in arg:
                no_max_comp = False
            if "--max-line-length" in arg:
                no_max_line = False

        if no_max_comp is True:
            args.append("--max-complexity=10")
        if no_max_line is True:
            args.append(f"--max-line-length={MAX_LINE_LENGTH}")

        super().__init__(args)

    def run(self):
        """Will run the flake8 linter."""
        args = [
            "flake8",
            consts.EXECUTED_FROM,
            *self._args,
        ]

        return _run(self.name, args)

    def check(self):
        """Will run the flake8 linter."""
        return self.run()


class ISort(Linter):
    """
    ISort linter.

    Sorts python imports.

    https://pycqa.github.io/isort/

    NOTE: Isort run WILL modify your files.
    """

    name: str = "isort"
    parallel_run: bool = False
    weight = 100
    language = Language.PYTHON

    def run(self):
        """
        Will run the isort linter.

        NOTE: This WILL modify your files.
        """
        args = [
            "isort",
            consts.EXECUTED_FROM,
            *self._args,
        ]
        return _run(self.name, args)

    def check(self):
        """
        Will run the isort linter in check mode.

        Note: This will NOT modify your files.
        """
        args = [
            "isort",
            "-c",
            consts.EXECUTED_FROM,
            *self._args,
        ]
        return _run(self.name + CHECK_STRING, args)


class Mccabe(Linter):
    """
    Flake8 runs Mccabe.

    Only use this if you do not also run Flake8
    """

    name: str = "mccabe"
    parallel_run: bool = True
    language = Language.PYTHON

    def run(self):
        """Will run the mccabe linter."""
        args = [
            "python",
            "-m",
            "mccabe",
            consts.EXECUTED_FROM,
            *self._args,
        ]
        return _run(self.name, args)

    def check(self):
        """Will run the mccabe linter."""
        self.run()


class Mypy(Linter):
    """
    Mypy linter.

    Python static type checker.
    https://mypy.readthedocs.io/en/stable/index.html
    """

    name: str = "mypy"
    parallel_run: bool = True
    language = Language.PYTHON

    def run(self):
        """Will run the mypy linter."""
        args = [
            "mypy",
            consts.EXECUTED_FROM,
            *self._args,
        ]
        return _run(self.name, args)

    def check(self):
        """Will run the mypy linter."""
        return self.run()


class PyDocStringFormatter(Linter):
    """
    PyDocStringFormatter linter.

    A tool to automatically format Python docstrings to follow recommendations from PEP 8 and PEP 257.
    https://pydocstringformatter.readthedocs.io/en/latest/index.html
    """

    name: str = "py_doc_string_formatter"
    parallel_run: bool = False
    weight = 98
    language = Language.PYTHON

    def run(self):
        """
        Will run the pydocstringformatter linter.

        NOTE: This WILL modify your files.
        """
        args = [
            "pydocstringformatter",
            "-w",
            consts.EXECUTED_FROM,
            *self._args,
        ]

        return _run(self.name, args)

    def check(self):
        """
        Will run the pydocstringformatter linter in check mode.

        NOTE: This will NOT modify your files.
        """
        args = [
            "pydocstringformatter",
            consts.EXECUTED_FROM,
            *self._args,
        ]

        return _run(self.name + CHECK_STRING, args)


class Pydocstyle(Linter):
    """
    Pydocstyle linter.

    Static analysis of docstrings to follow PEP 257.
    https://www.pydocstyle.org/en/stable/
    """

    name: str = "pydocstyle"
    parallel_run: bool = True
    language = Language.PYTHON

    def run(self):
        """Will run the pydocstyle linter."""
        args = [
            "pydocstyle",
            consts.EXECUTED_FROM,
            *self._args,
        ]
        return _run(self.name, args)

    def check(self):
        """Will run the pydocstyle linter."""
        return self.run()


class Pyflakes(Linter):
    """
    Flake8 runs pyflakes.

    Only use this if you do not also run Flake8
    """

    name: str = "pyflakes"
    parallel_run: bool = True
    language = Language.PYTHON

    def run(self):
        """Will run the pyflakes linter."""
        args = [
            "pyflakes",
            consts.EXECUTED_FROM,
            *self._args,
        ]
        return _run(self.name, args)

    def check(self):
        """Will run the pyflakes linter."""
        return self.run()


class Pylint(Linter):
    """
    Pylint linter.

    Static code analysis for Python. Checks for errors, code smells, and formatting.
    https://pylint.pycqa.org/en/latest/
    """

    name: str = "pylint"
    parallel_run: bool = True
    language = Language.PYTHON

    def run(self):
        """Will run the pylint linter."""
        args = [
            "pylint",
            consts.EXECUTED_FROM,
            *self._args,
        ]
        return _run(self.name, args)

    def check(self):
        """Will run the pylint linter."""
        return self.run()


class CheckOV(Linter):
    """
    CheckOV linter.

    Cloud infrastructure configurations to find misconfigurations.
    https://www.checkov.io/1.Welcome/Quick%20Start.html

    NOTE: This will ALWAYS run in a docker container. CheckOV will not be installed on the system.
    """

    name: str = "checkov"
    parallel_run: bool = True
    language = Language.TERRAFORM

    def run(self):
        """Will run the checkov linter in a docker container."""
        args = [
            "docker",
            "run",
            "--tty",
            "--rm",
            "--volume",
            f"{consts.EXECUTED_FROM}:/tf",
            "--workdir",
            "/tf",
            "bridgecrew/checkov",
            "--directory",
            "/tf",
            *self._args,
        ]
        return _run(self.name, args)

    def check(self):
        """Will run the checkov linter in a docker container."""
        return self.run()


class TerrascanTerraform(Linter):
    """
    Terrascan linter.

    Only running on Terraform files

    Detect compliance and security violations across Infrastructure as Code
    https://runterrascan.io/docs/

    NOTE: This will ALWAYS run in a docker container. Terrascan will not be installed on the system.
    """

    name: str = "terrascan_terraform"
    parallel_run: bool = True
    language = Language.TERRAFORM

    def run(self):
        """Will run the terrascan linter for terraform in a docker container."""
        args = [
            "docker",
            "run",
            "--rm",
            "--volume",
            f"{consts.EXECUTED_FROM}:/iac",
            "--workdir",
            "/iac",
            "tenable/terrascan",
            "scan",
            "-i",
            "terraform",
            *self._args,
        ]
        return _run(self.name, args)

    def check(self):
        """Will run the terrascan linter for terraform in a docker container."""
        return self.run()


class TFLint(Linter):
    """
    TFLint linter.

    Find possible errors (like illegal instance types) for Major Cloud providers
    https://github.com/terraform-linters/tflint

    WARNING: TFLint always checks only the current root module (no recursive check). This makes it a pain
    to work with.

    NOTE: This will ALWAYS run in a docker container. TFLint will not be installed on the system.
    """

    name: str = "tflint"
    parallel_run: bool = True
    language = Language.TERRAFORM

    def run(self):
        """Will run the tflint linter in a docker container."""
        args = [
            "docker",
            "run",
            "--rm",
            "-v",
            f"{consts.EXECUTED_FROM}:/data",
            "-t",
            "ghcr.io/terraform-linters/tflint",
            *self._args,
        ]
        return _run(self.name, args)

    def check(self):
        """Will run the tflint linter in a docker container."""
        return self.run()


class TFSec(Linter):
    """
    TFSec linter.

    Static analysis security scanner for Terraform
    https://aquasecurity.github.io/tfsec/v1.27.6/

    NOTE: This will ALWAYS run in a docker container. TFsec will not be installed on the system.
    """

    name: str = "tfsec"
    parallel_run: bool = True
    language = Language.TERRAFORM

    def run(self):
        """Will run the tfsec linter in a docker container."""
        args = [
            "docker",
            "run",
            "--rm",
            "-it",
            "-v",
            f"{consts.EXECUTED_FROM}:/src",
            "aquasec/tfsec",
            "/src",
            *self._args,
        ]
        return _run(self.name, args)

    def check(self):
        """Will run the tfsec linter in a docker container."""
        return self.run()


class TerraformFmt(Linter):
    """
    Terraform fmt linter.

    https://www.terraform.io/cli/commands/fmt

    NOTE: This will ALWAYS run in a docker container. Terraform will not be installed on the system.
    """

    name: str = "terraform_fmt"
    parallel_run: bool = False
    weight = 100
    language = Language.TERRAFORM

    def run(self):
        """Will run the terraform fmt linter in a docker container."""
        args = [
            "docker",
            "run",
            "--rm",
            "--volume",
            f"{consts.EXECUTED_FROM}:/opt",
            "--workdir",
            "/opt",
            "hashicorp/terraform",
            "fmt",
            *self._args,
        ]
        return _run(self.name, args)

    def check(self):
        """Will run the terraform fmt check linter in a docker container."""
        args = [
            "docker",
            "run",
            "--rm",
            "--volume",
            f"{consts.EXECUTED_FROM}:/opt",
            "--workdir",
            "/opt",
            "hashicorp/terraform",
            "fmt",
            "-check",
            *self._args,
        ]
        return _run(self.name, args)


class TerrascanDocker(Linter):
    """
    Terrascan linter.

    Only for Dockerfiles.

    Detect compliance and security violations across Infrastructure as Code
    https://runterrascan.io/docs/

    NOTE: This will ALWAYS run in a docker container. Terrascan will not be installed on the system.
    """

    name: str = "terrascan_docker"
    parallel_run: bool = True
    language = Language.TERRAFORM

    def run(self):
        """Will run the terrascan linter for dockerfiles in a docker container."""
        args = [
            "docker",
            "run",
            "--rm",
            "--volume",
            f"{consts.EXECUTED_FROM}:/iac",
            "--workdir",
            "/iac",
            "tenable/terrascan",
            "scan",
            "-i",
            "docker",
            *self._args,
        ]
        return _run(self.name, args)

    def check(self):
        """Will run the terrascan linter for dockerfiles in a docker container."""
        return self.run()


DEFAULT_PYTHON = [
    ISort(),
    Black(args=[f"--line-length={MAX_LINE_LENGTH}"]),
    PyDocStringFormatter(),
    Bandit(),
    Flake8(args=["--ignore=E203", "--ignore=W503"]),
    Mypy(),
    Pydocstyle(),
    Pylint(args=[f"--max-line-length={MAX_LINE_LENGTH}"]),
]

DEFAULT_TERRAFORM = [
    TerraformFmt(args=["./terraform"]),
    CheckOV(),
    TerrascanTerraform(args=["-d", "./terraform"]),
    TFSec(),
]

DEFAULT_DOCKER = [
    TerrascanDocker(),
]



"""Contains the interfaces that backends will implement."""
import abc
import typing as t


# pylint: disable=R0801
class Backend(abc.ABC):
    """The interface that all backends will implement."""

    name: str

    @abc.abstractmethod
    def build(self, force_rebuild: bool = False):
        """Will run any build jobs that are needed for the backend."""
        raise NotImplementedError

    @abc.abstractmethod
    def purge(self):
        """Will remove any data written by the backend."""
        raise NotImplementedError

    @abc.abstractmethod
    def run(self, args: t.List[str], workdir: str) -> t.Tuple[int, str]:
        """Will run a command in the backend."""
        raise NotImplementedError

    @abc.abstractmethod
    def interactive(self):
        """Will drop the user into an interactive shell."""
        raise NotImplementedError

    @abc.abstractmethod
    def shutdown(self):
        """Will shut down any instances of the backend."""
        raise NotImplementedError



"""Contains all the implemented testers."""
import typing as t

from py_mono_tools.config import consts, GREEN, logger, RED, RESET
from py_mono_tools.goals.interface import Language, Tester


def _run(tester: str, args: t.List[str], workdir=None) -> t.Tuple[str, int]:
    log_format = "\n" + "#" * 20 + "  {}  " + "#" * 20 + "\n"
    logs = ""

    logger.debug("Running %s: %s", tester, args)
    if len(args) > 0 and "docker" == args[0] and consts.CURRENT_BACKEND.name == "docker":  # type: ignore
        logger.debug("Bypassing docker backend for system backend. Tester: %s", tester)
        return_code, returned_logs = consts.BACKENDS["system"]().run(args, workdir=workdir)  # type: ignore
    else:
        return_code, returned_logs = consts.CURRENT_BACKEND.run(args, workdir=workdir)  # type: ignore
    logger.debug("%s return code: %s", tester, return_code)

    color = GREEN if return_code == 0 else RED
    logs += color
    logs += log_format.format(tester + " start")
    logs = logs + returned_logs
    logs += color
    logs += log_format.format(tester + " end")
    logs += RESET

    return logs, return_code


class PytestTester(Tester):
    """Pytest tester."""

    name = "pytest"
    language = Language.PYTHON

    def run(self):
        """Will Run pytest.

        Changes working dir to the workdir passed to the class init.
        """
        args = [
            "pytest",
        ]

        return _run(self.name, args, workdir=self._test_dir)



"""Contains all the commands that the CLI can execute."""
import importlib
import importlib.machinery
import importlib.util
import inspect
import logging
import os
import os.path
import pathlib
import sys
import typing as t
from types import ModuleType

import click

from py_mono_tools.backends import Docker, System
from py_mono_tools.config import consts, logger
from py_mono_tools.goals import linters as linters_mod
from py_mono_tools.goals.interface import Language, Linter


logger.info("Starting main")


def _find_linters():
    logger.debug("Finding linters")
    linter_classes = inspect.getmembers(linters_mod, inspect.isclass)
    linter_instances = [
        linter_class[1]()
        for linter_class in linter_classes
        if issubclass(linter_class[1], Linter) and linter_class[1] != Linter
    ]
    # pylint: disable=invalid-name
    consts.ALL_LINTERS = linter_instances

    for linter in linter_instances:
        consts.ALL_LINTER_NAMES.append(linter.name)

    logger.debug("Found linters: %s", consts.ALL_LINTER_NAMES)


_find_linters()


def _load_conf(conf_path: str) -> ModuleType:
    conf_location = f"{conf_path}/CONF"
    logger.debug("CONF location: %s", conf_location)

    loader = importlib.machinery.SourceFileLoader("CONF", conf_location)
    spec = importlib.util.spec_from_loader(loader.name, loader)
    mod = importlib.util.module_from_spec(spec)  # type: ignore
    loader.exec_module(mod)

    return mod


def _init_logger(verbose: bool):
    if verbose:
        logger.setLevel(logging.DEBUG)
        for handler in logger.handlers:
            handler.setLevel(logging.DEBUG)


def _set_absolute_path(absolute_path: str):
    logger.info("Overwriting execution root path: %s", absolute_path)

    # pylint: disable=invalid-name
    consts.EXECUTED_FROM = pathlib.Path(absolute_path).resolve()

    os.chdir(consts.EXECUTED_FROM.resolve())  # pylint: disable=invalid-name


def _set_relative_path(relative_path: str):
    logger.info("Overwriting execution root path: %s", relative_path)

    # pylint: disable=invalid-name
    consts.EXECUTED_FROM = consts.EXECUTED_FROM.joinpath(pathlib.Path(relative_path).resolve())
    os.chdir(consts.EXECUTED_FROM)


def _set_path_from_conf_name(name: str):
    logger.info("Setting path from conf name: %s", name)

    for rel_path, _, filenames in os.walk("."):
        for filename in filenames:
            if filename == "CONF":
                path = pathlib.Path(rel_path).resolve()
                mod = _load_conf(str(path))
                logger.debug("Found CONF: %s in %s", mod.NAME, path)
                if mod.NAME.strip().lower() == name.strip().lower():
                    logger.debug("Using CONF: %s in %s", mod.NAME, path)
                    _set_absolute_path(str(path))
                    return


def _init_backend(_build_system: str):
    logger.debug("Initializing build system: %s", _build_system)

    # pylint: disable=invalid-name
    consts.BACKENDS = {
        Docker.name: Docker,
        System.name: System,
    }

    consts.CURRENT_BACKEND = consts.BACKENDS[_build_system]()


def _filter_linters(specific_linters: t.List[str], language: t.Optional[Language]) -> t.List[Linter]:
    conf_linters: t.List[Linter] = consts.CONF.LINT  # type: ignore
    logger.debug("Linters: %s", conf_linters)
    linters_to_run: t.List[Linter] = []
    specific_linters = [s.strip().lower() for s in specific_linters]

    if language is None and not specific_linters:
        logger.debug("No language or specific linters specified. Running all linters.")
        linters_to_run = conf_linters
    else:
        logger.debug("Language: %s, specific: %s", language, specific_linters)
        for linter in conf_linters:
            if linter.name.strip().lower() in specific_linters:
                linters_to_run.append(linter)
                continue
            if linter.language == language:
                linters_to_run.append(linter)

    logger.debug("Linters to run: %s", linters_to_run)
    return linters_to_run


@click.group()
@click.option(
    "--backend",
    default=None,
    type=str,
    help="""
Default backend is \"system\".
This can be set via this flag, the BACKEND var in CONF, or defaulted to system.
""",
)
@click.option("--absolute_path", "-ap", default=None, type=click.Path())
@click.option("--relative_path", "-rp", default=None, type=click.Path())
@click.option("--name", "-n", default=None, type=str, help="Name as defined in CONF NAME=...")
@click.option("--verbose", "-v", default=False, is_flag=True)
def cli(backend, absolute_path, relative_path, name, verbose):
    """Py mono tool is a CLI tool that simplifies using python in a monorepo."""
    logger.info("Starting cli")

    if "--help" in sys.argv or "-h" in sys.argv:
        return

    _init_logger(verbose=verbose)

    logger.debug("Executed from: %s", consts.EXECUTED_FROM)
    logger.debug("Current backend: %s", consts.CURRENT_BACKEND)
    logger.debug("Backends: %s", consts.BACKENDS)

    if absolute_path is not None:
        _set_absolute_path(absolute_path=absolute_path)
    elif relative_path is not None:
        _set_relative_path(relative_path=relative_path)
    elif name is not None:
        _set_path_from_conf_name(name)

    mod = _load_conf(consts.EXECUTED_FROM)

    # pylint: disable=invalid-name
    consts.CONF = mod
    if backend is None:
        try:
            backend = consts.CONF.BACKEND
        except AttributeError:
            backend = "system"

    logger.info("Using backed: %s", backend)

    _init_backend(backend)


@cli.command()
@click.option("--check", is_flag=True, default=False)
@click.option(
    "--specific",
    "-s",
    multiple=True,
    default=[],
    help=f"""
    Specify one or more linters to run. NOTE: The linter MUST be listed in the respected CONF file.

    All Linters:
    {consts.ALL_LINTER_NAMES}
    """,
    shell_complete=consts.ALL_LINTER_NAMES,
)
@click.option("--fail_fast", "-ff", is_flag=True, default=False, help="Stop on first failure.")
@click.option("--show_success", is_flag=True, default=False, help="Show successful outputs")
@click.option(
    "--parallel",
    is_flag=True,
    default=False,
    help="""
    NOT IMPLEMENTED
    Runs all linters marked with parallel_run=True at the same time
    NOTE: All linters labeled as parallel_run=False will be run BEFORE ones marked as True.
    """,
)
@click.option(
    "--ignore_linter_weight", is_flag=True, default=False, help="Ignores linter weight and runs in the order in CONF."
)
@click.option("--language", "-l", default=None, type=Language, help="Specify a language to run linters for.")
def lint(
    check: bool,
    specific: t.List[str],
    fail_fast: bool,
    show_success: bool,
    parallel: bool,
    ignore_linter_weight: bool,
    language: t.Optional[Language],
):  # pylint: disable=too-many-arguments
    """
    Run one or more Linters specified in the CONF file.

    Examples:
    ```bash
    pmt lint
    pmt lint -s black -s flake8
    pmt -rp ./some/path lint
    pmt -n py_mono_tools lint -l python
    ```
    """
    if parallel is True:
        raise NotImplementedError

    logger.info("Starting lint")

    linters_to_run = _filter_linters(specific_linters=specific, language=language)

    if ignore_linter_weight is False:
        linters_to_run.sort(key=lambda x: x.weight, reverse=True)

    for linter in linters_to_run:
        logger.debug("Linting: %s", linter)
        if check is True:
            logs, return_code = linter.check()
        else:
            logs, return_code = linter.run()
        logger.info("Lint result: %s %s", linter.name, return_code)

        if show_success is False and return_code == 0:
            logger.debug("Skipping successful output")
            logger.debug(logs)
        else:
            logger.info(logs)

        if fail_fast is True and return_code != 0:
            logger.error("Linter %s failed with code %s", linter.name, return_code)
            sys.exit(1)

    logger.info("Linting complete")


@cli.command()
def test():
    """Run all the tests specified in the CONF file."""
    testers = consts.CONF.TEST
    for tester in testers:
        logger.info("Testing: %s", tester.name)
        logs, return_code = tester.run()
        logger.info("Test result: %s %s", tester.name, return_code)
        logger.info(logs)


@cli.command()
def deploy():
    """Run the specified build and deploy in the specific CONF file."""
    deployers = consts.CONF.DEPLOY
    for deployer in deployers:
        logger.info("Deploying: %s", deployer.name)
        logs, return_code = deployer.plan()
        logger.info("Deploy result: %s %s", deployer.name, return_code)
        logger.info(logs)


# @cli.command()
# @click.option("--force-rebuild", is_flag=True, default=False)
# @click.option("--modules", multiple=True, type=t.List[str], default=["all"])
# def build(force_rebuild, modules):
#     """
#     Run the build process for the specified build system.
#
#     Docker is the default.
#     """
#     logger.info(modules)
#     consts.CURRENT_BACKEND.build(force_rebuild=force_rebuild)
#
#
#
#
# @cli.command()
# def init():
#     """Initialize the current directory with the necessary files."""
#     raise NotImplementedError
#
#
# @cli.group()
# def new():
#     """Create a new module with the specified name."""
#     raise NotImplementedError
#
#
# @new.command()
# def migration():
#     """Generate an Alembic Database migration."""
#     raise NotImplementedError
#
#
# @new.command()
# def package():
#     """Not implemented."""
#     raise NotImplementedError
#
#
# @cli.command()
# def run():
#     """Not implemented."""
#     raise NotImplementedError
#
#
# @cli.command()
# def interactive():
#     """Drop into an interactive session in your specified backend."""
#     raise NotImplementedError
#
#
# @cli.command()
# def setup():
#     """Not implemented."""
#     raise NotImplementedError
#
#
# @cli.command()
# def validate():
#     """Not implemented."""
#     raise NotImplementedError



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['py_mono_tools', 'py_mono_tools.backends', 'py_mono_tools.goals']

package_data = \
{'': ['*'], 'py_mono_tools': ['templates/*', 'templates/dockerfiles/*']}

install_requires = \
['click>=8,<9', 'pydocstyle[toml]>=6.1.1,<7.0.0', 'pytest>=7.1.3,<8.0.0']

extras_require = \
{':extra == "linters-python"': ['bandit>=1.7.4,<2.0.0',
                                'black>=22.8.0,<23.0.0',
                                'flake8>=5.0.4,<6.0.0',
                                'isort>=5.10.1,<6.0.0',
                                'mypy>=0.971,<0.972',
                                'pydocstringformatter>=0.7.2,<0.8.0',
                                'pylint>=2.15.2,<3.0.0']}

entry_points = \
{'console_scripts': ['pmt = py_mono_tools.main:cli',
                     'py_mono_tools = py_mono_tools.main:cli']}

setup_kwargs = {
    'name': 'py-mono-tools',
    'version': '0.0.21',
    'description': 'A CLI designed to make it easier to work in a python mono repo',
    'long_description': 'For more information, please go the GitHub page. https://peterhoburg.github.io/py_mono_tools/\n',
    'author': 'Peter Hoburg',
    'author_email': 'peterHoburg@users.noreply.github.com',
    'maintainer': 'Peter Hoburg',
    'maintainer_email': 'peterHoburg@users.noreply.github.com',
    'url': 'https://peterhoburg.github.io/py_mono_tools/',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import pathlib
import subprocess

from py_mono_tools.config import consts, logger
from py_mono_tools.goals.interface import Deployers, Language


class PoetryBuilder(Deployers):
    """Class to interact with terraform."""

    name: str = "terraform"
    language = Language.PYTHON

    def _run_poetry(self, commands: list):
        logger.info("running command: %s", commands)

        cwd = consts.EXECUTED_FROM
        cwd = cwd / pathlib.Path(self._pyproject_loc).parent
        cwd = cwd.resolve()
        logger.info("cwd: %s", cwd)

        with subprocess.Popen(  # nosec B603
            commands,
            cwd=cwd,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        ) as process:
            stdout_data, stderr_data = process.communicate()

        return process.returncode, stderr_data.decode("utf-8") + stdout_data.decode("utf-8")

    def plan(self):
        return self.run(dry_run=True)

    def build(self):
        commands = [
            "poetry",
            "build",
        ]
        return self._run_poetry(commands)

    def run(self, dry_run: bool = False):
        return_code, build_logs = self.build()

        logger.debug("build_return_code: %s build logs: %s", return_code, build_logs)

        if return_code != 0:
            logger.error("build failed: %s", build_logs)
            return return_code, build_logs

        commands = [
            "poetry",
            "publish",
        ]
        if dry_run is True:
            commands.append("--dry-run")

        return_code, run_logs = self._run_poetry(commands)
        logger.debug("run_return_code: %s run logs: %s", return_code, run_logs)

        return return_code, build_logs + run_logs
